# -*- coding: UTF-8 -*-
import torch
import torch.autograd as autograd
import torch.nn as nn
import torch.nn.functional as F
import torch.optim as optim
import numpy as np
from torch.utils.data import DataLoader, TensorDataset
import pandas as pd
class CharRNN(nn.Module):
    def __init__(self, embedding_dim, hidden_dim, num_layers, vocab_size, batch_size):
        super(CharRNN, self).__init__()
        self.hidden_dim = hidden_dim
        self.num_layers = num_layers
        self.batch_size = batch_size
        self.vocab_size = vocab_size
        self.word_embeddings = nn.Embedding(vocab_size, embedding_dim)
        self.lstm = nn.LSTM(embedding_dim, hidden_dim,num_layers,batch_first=True)
        self.hidden2tag = nn.Linear(hidden_dim, vocab_size)
    def init_hidden(self):
        return (autograd.Variable(torch.zeros(self.num_layers, self.batch_size, self.hidden_dim)),
                autograd.Variable(torch.zeros(self.num_layers, self.batch_size, self.hidden_dim)))

    def forward(self, x, training):
        if training:
            self.hidden = (autograd.Variable(torch.zeros(self.num_layers, self.batch_size, self.hidden_dim)),
                autograd.Variable(torch.zeros(self.num_layers, self.batch_size, self.hidden_dim)))
            embeds = self.word_embeddings(x)
            lstm_out, self.hidden = self.lstm(embeds, self.hidden)
            output = self.hidden2tag(lstm_out.contiguous().view(-1, self.hidden_dim))
        else:
            self.hidden = (autograd.Variable(torch.zeros(self.num_layers, 717, self.hidden_dim)),
                           autograd.Variable(torch.zeros(self.num_layers, 717, self.hidden_dim)))
            embeds = self.word_embeddings(x)
            lstm_out, self.hidden = self.lstm(embeds, self.hidden)
            output = self.hidden2tag(lstm_out[:,-1,:].contiguous().view(-1, self.hidden_dim))
        return output
training_data=[]
data = np.loadtxt('seq_train_120_480_padding.csv',delimiter=",", dtype=np.int)
fileWriter = open(r"E:\indoorLocationPrediction\codes\myrnn\acc.csv", mode="w")
trainData = data[:1600, :]
testData = data[1600:, :]
x_train = torch.LongTensor(trainData[:, :-1])
y_train = torch.LongTensor(trainData[:, 1:])
x_test = torch.LongTensor(testData[:, :-1])
y_test = testData[:, 1:]
ds = TensorDataset(x_train, y_train)
train_loader = DataLoader(dataset=ds, batch_size=100, shuffle=True)
model = CharRNN(64, 256, 1, 490,100)
optimizer = torch.optim.Adam(model.parameters(), lr=1e-3)
criterion = nn.CrossEntropyLoss()


def printAccuracy(out, y_test, k, fileWriter):
    accuracy = []
    res = np.argsort(out, axis=1)
    y_test = y_test[:,-1].reshape(-1)
    for i in range(len(y_test)):
        for j in range(-k, 0):
            if res[i,j] == y_test[i]:
                accuracy.append(1)
                break
    print("预测正确率：%s" % (sum(accuracy) / len(y_test),))
    fileWriter.write("%s," % (sum(accuracy) / len(y_test),))
    fileWriter.flush()


while True:
    for index, (batch_x, batch_y) in enumerate(train_loader):
        model.zero_grad()
        pred = model(batch_x, True)
        loss = criterion(pred, batch_y.view(-1))
        loss.backward()
        optimizer.step()
        if index % 50 == 0:
            test_output = model(x_test, False)
            printAccuracy(test_output.data.numpy(),y_test, k=1, fileWriter=fileWriter)
